function out = GPDEMMPDE(n,tspan,mmpdenumber,tau,smoothing,rh,param,rtol,atol,clmatcont,odesolver,impl)

%--------------------------------------------------------------------------
% This is a general solver for the following two problems:
%   1) applying moving meshes for PDE's of the form
%      u_t = f(u,u_x,u_xx,...), where u_t denotes the partial derivative of u
%      with respect to t, on the specified time interval tspan, x element
%      of [0,1]. With boundary and intial conditions specified by rh.
%
%   2) Solving MMPDE for a "known" solution function u(x,t), where u_x is
%   specified in the rh - function.
%--------------------------------------------------------------------------
%INPUT
%n is the number of uniformly distributed nodes on [0,1] 
%
%rh is a file specifing in case of (1) a discrete version of the right hand side of the pde, and
%in case of (2) the functions u and u_x. rh also in both cases contain functions for
%boundary and initial conditions. 
%
%tspan is the time interval for which the solution is to be calculated
%
%mmpdenumber refer to the MMPDE to be used for the mesh movement. The
%number refers to the corresponding MMPDE in Huang, Ren & Russel
%"Moving mesh partial differential equation (MMPDES) based on the
%equidistribution principle". 
%
%tau refer to the relaxation constant in the MMPDES
%
%smoothing is a boolean which is true (1) if we want to smooth our monitor
%function.
%
%param is a vector of the parameters for the active function, suggestions
%for these can be found in the rh file.
%
%rtol - relative tolerance
%
%atol - absolute tolerance
%
%clmatcont -boolean: true if clmatcont's ode solvers are used
%
%odesolver -functionhandle to the odesolver to be used 
%
%impl - true if the odesolver needs implicit formulation
%--------------------------------------------------------------------------
%--------------------------------------------------------------------------
%Author: Eirik Ravns - december 2007
%--------------------------------------------------------------------------

global N; %number of internal nodes
global funct;%the function rh
global TAU; %relaxation parameter
global smooth; %boolean: smoothing or not
global PDE; %boolean: do we solve a pde (case 1) or not (case(2)).
global x0; %x0
global xNP1; %x-end
global libfile;
global v;

libfile = MMPDELIB;
v = param;

x0 = 0; %obs skal hentes
xNP1 = 1; %obs skal hentes

smooth= smoothing;
TAU = tau;
funct = rh;
N = n;

H=0:(1/(N+1)):1; %grid on the computational domain
%check weather we have a pde or not...
if feval(funct{5}) == 0;
    PDE = 0;
else
    PDE =1;
end

%sparsity pattern
%JP = JPat(N) | MvPat(N);

%Initial values
if PDE
    xInit = feval(funct{7},H(2:end-1)');
    u0 = feval(funct{4},xInit);
    y0 = [u0;xInit];
    yp0 = zeros(size(y0));
else
    y0 = feval(funct{7},H(2:end-1)'); %here y=x..., i.e initial values for all internal nodes
    yp0 = zeros(size(y0));
end
 
switch mmpdenumber
     case {4}
        disp('Using MMPDE4');
        if clmatcont
            opts = odeset('Mass',@mass4,'RelTol',rtol,'AbsTol',atol);
            
        else
            opts = odeset('Mass',@mass4,'MStateDep','weak','RelTol',rtol,'AbsTol',atol);%,'JPattern',JPat(N),'MvPattern',MvPat(N));%,'Jacobian',@Jac,'JPattern',{JP,[]});        
                    
        end
        if impl
            [y0,yp0] = decic(@Impl4, tspan(1),y0,[],yp0,[],opts);
            %solving
            out = odesolver(@Impl4,tspan,y0,yp0,opts);
        else
            out = odesolver(@G,tspan,y0,opts);
        end
    case {5}
        disp('Using MMPDE5');
        if clmatcont
            opts = odeset('Mass',@mass5,'RelTol',rtol,'AbsTol',atol);
        else
            opts = odeset('Mass',@mass5,'MStateDep','weak','RelTol',rtol,'AbsTol',atol);%,'Jacobian',@Jac,'JPattern',{JP,[]});        
        end
        %compute consistent initial conditions
        if impl
            [y0,yp0] = decic(@Impl5, tspan(1),y0,[],yp0,[],opts);
            %solving
            out = odesolver(@Impl5,tspan,y0,yp0,opts);
        else
            out = odesolver(@G,tspan,y0,opts);
        end
    case{6}
        disp('Using MMPDE6');
        if clmatcont
            opts = odeset('Mass',@mass6,'RelTol',rtol,'AbsTol',atol);
        else
            opts = odeset('Mass',@mass6,'MStateDep','weak','RelTol',rtol,'AbsTol',atol);%,'Jacobian',@Jac,'Jpattern',{JP,[]});        
            %opts = odeset('RelTol',1e-5,'AbsTol',1e-4,'Jacobian',@Jac,'Jpattern',{JP,[]});
        end
        if impl
            [y0,yp0] = decic(@Impl6, tspan(1),y0,[],yp0,[],opts);
            %solving
            out = odesolver(@Impl6,tspan,y0,yp0,opts);
        else
            out = odesolver(@G,tspan,y0,opts);
        end
    case {7}
        disp('Using MMPDE7');
        if clmatcont
            opts = odeset('Mass',@mass7,'RelTol',rtol,'AbsTol',atol);
        else
            opts = odeset('Mass',@mass7,'MStateDep','weak','RelTol',rtol,'AbsTol',atol);%,'Jacobian',@Jac,'JPattern',{JP,[]});
        end
        if impl
            [y0,yp0] = decic(@Impl7, tspan(1),y0,[],yp0,[],opts);
            %solving
            out = odesolver(@Impl7,tspan,y0,yp0,opts);
        else
            out = odesolver(@G,tspan,y0,opts);
        end
    otherwise
        disp('Wrong MMPDE number');
        
end


end %GPDEMMPDE

%--------------------------------------------------------------------------
%nested functions

function res4 = Impl4(t,y,yp)
    res4 = mass4(t,y)*yp-G(t,y);
end %Impl4

function res5 = Impl5(t,y,yp)
    res5 = mass5(t,y)*yp-G(t,y);
end %Impl5

function res6 = Impl6(t,y,yp)
    res6 = mass6(t,y)*yp-G(t,y);
end %Impl6

function res7 = Impl7(t,y,yp)
    res7 = mass7(t,y)*yp-G(t,y);
end %Impl7

function ut = G(t,y) %the right hand side of the system
    global N;
    global funct;
    global TAU;
    global smooth;
    global PDE;
    global x0;
    global xNP1;
    global libfile;
    global v;
    ut = feval(libfile{1},t,y,N,funct,v,TAU,smooth,PDE,x0,xNP1);
    
end %G

%%Mass Matrices%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%mass matrix for MMPDE4;
function ans4 = mass4(t,y)
global N; %number of internal nodes
global funct;%the function rh
%global TAU; %relaxation parameter
global smooth; %boolean: smoothing or not
global PDE; %boolean: do we solve a pde (case 1) or not (case(2)).
global x0; %x0
global xNP1; %x-end
global libfile;
global v;
ans4 = feval(libfile{2},t,y,N,funct,v,PDE,smooth,x0,xNP1);
end%mass4

%Mass matrix for MMPDE5
function ans5 = mass5(t,y)
global N; %number of internal nodes
global funct;%the function rh
%global TAU; %relaxation parameter
global smooth; %boolean: smoothing or not
global PDE; %boolean: do we solve a pde (case 1) or not (case(2)).
global x0; %x0
global xNP1; %x-end
global libfile;
global v;
ans5 = feval(libfile{3},t,y,N,funct,v,PDE,smooth,x0,xNP1);
end %mass5

%mass matrix for MMPDE6;
function ans6 = mass6(t,y)
global N; %number of internal nodes
global funct;%the function rh
%global TAU; %relaxation parameter
global smooth; %boolean: smoothing or not
global PDE; %boolean: do we solve a pde (case 1) or not (case(2)).
global x0; %x0
global xNP1; %x-end
global libfile;
global v;
ans6 = feval(libfile{4},t,y,N,funct,v,PDE,smooth,x0,xNP1);
end %mass6

function ans7 = mass7(t,y)
global N; %number of internal nodes
global funct;%the function rh
%global TAU; %relaxation parameter
global smooth; %boolean: smoothing or not
global PDE; %boolean: do we solve a pde (case 1) or not (case(2)).
global x0; %x0
global xNP1; %x-end
global libfile;
global v;    
ans7 = feval(libfile{5},t,y,N,funct,v,PDE,smooth,x0,xNP1);    
end%mass7

% Sparsity
% Patterns%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function out = JPat(N)
% Jacobian sparsity pattern
  S1 = spdiags(ones(N,3),-1:1,N,N);
  S2 = spdiags(ones(N,9),-4:4,N,N);
  out = [S1 S1
         S2 S2];
end  
% -------------------------------------------------------------------------   
   
function S = MvPat(N)
% Sparsity pattern for the derivative of the Mass matrix times a vector
  S = sparse(2*N,2*N);
  S(1,2) = 1;
  S(1,2+N) = 1;
  for i = 2:N-1
    S(i,i-1) = 1;
    S(i,i+1) = 1;
    S(i,i-1+N) = 1;
    S(i,i+1+N) = 1;
  end
  S(N,N-1) = 1;
  S(N,N-1+N) = 1;
end  
  
